/******************************************************************************
 * Copyright (c) 2013-2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#pragma omp declare target
#include <omp.h>
#include <stdio.h>
#include <stdint.h>
extern uint64_t __clock64(void);
#pragma omp declare target end

void null()
{
    double host_start = 0, host_end = 0, host_elapsed = 0;
    uint64_t device_start = 0, device_end = 0, device_elapsed_cyc = 0;
    host_start = omp_get_wtime();
    #pragma omp target map(from:device_start, device_end)
    {
        /* Get cycle count */
        device_start = __clock64();
        
        int i, j=0;
        #pragma omp parallel for private(i) firstprivate(j)
        for (i = 0; i<80; i++)
            j++;
        
        /* Get cycle count */
        device_end = __clock64();
    }
    host_end = omp_get_wtime();
    
    host_elapsed = host_end - host_start;

    /* This gives us the elapsed cycles */
    device_elapsed_cyc = device_end - device_start;
    /* Assuming the clock speed of the DSP is 1.0e9 i.e. 1Ghz 
     * Calculate the elapsed seconds */
    double device_elapsed = device_elapsed_cyc * 1.0e-9;

    printf("Host launch elapsed time: %f\n", host_elapsed);
    printf("Device launch elapsed time: %f\n", device_elapsed);
    printf("Launch Overhead: %f\n", host_elapsed - device_elapsed);
}
