/******************************************************************************
 * Copyright (c) 2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
/**
 * \file omp.h
 *
 * \brief OMP header file to include in OpenMPAcc user applications
 */

#ifndef OMP_H_
#define OMP_H_

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/* Lock type */
typedef struct _omp_lock
{
    volatile int  entered;
    void         *owner_task;
    volatile int  count;
    int           hw_sem;
} *omp_lock_t;

typedef struct _omp_lock *omp_nest_lock_t;

/* Schedule kinds */
typedef enum omp_sched_t
{
  omp_sched_static = 1,
  omp_sched_dynamic = 2,
  omp_sched_guided = 3,
  omp_sched_auto = 4
} omp_sched_t;

/* Execution environment routines */
void omp_set_num_threads (int nthreads);
int omp_get_num_threads (void);
int omp_get_max_threads (void);
int omp_get_thread_num (void);
int omp_get_num_procs(void);
int omp_in_parallel(void);
void omp_set_dynamic (int nthreads);
int omp_get_dynamic (void);
void omp_set_nested (int val);
int omp_get_nested (void);
void omp_set_schedule (omp_sched_t kind, int modifier);
void omp_get_schedule (omp_sched_t *kind, int *modifier);
int omp_get_thread_limit (void);
void omp_set_max_active_levels(int val);
int omp_get_max_active_levels(void);
int omp_get_level (void);
int omp_get_ancestor_thread_num (int level);
int omp_get_team_size (int level);
int omp_get_active_level (void);
#if 0
int omp_in_final (void);
#endif

/* Lock Routines */
void omp_init_lock (omp_lock_t* lock);
void omp_destroy_lock (omp_lock_t *lock);
void omp_set_lock (omp_lock_t* lock);
int  omp_test_lock (omp_lock_t* lock);
void omp_unset_lock (omp_lock_t* lock);

void omp_init_nest_lock (omp_nest_lock_t* lock);
void omp_destroy_nest_lock (omp_nest_lock_t *lock);
void omp_set_nest_lock (omp_nest_lock_t* lock);
int  omp_test_nest_lock (omp_nest_lock_t* lock);
void omp_unset_nest_lock (omp_nest_lock_t* lock);

/* Timing Routines */
double omp_get_wtime (void);
double omp_get_wtick (void);

/* DSP Heap Management API for use with the accelerator model */
typedef int ti_omp_dsp_heap_idx;

void  __ti_omp_dsp_heap_init(ti_omp_dsp_heap_idx idx, char *ptr, int size);
void *__ti_omp_dsp_malloc   (ti_omp_dsp_heap_idx idx, int size);
void *__ti_omp_dsp_realloc  (ti_omp_dsp_heap_idx idx, void *ptr, int size);
void  __ti_omp_dsp_free     (ti_omp_dsp_heap_idx idx, void *ptr);
void *__ti_omp_dsp_calloc   (ti_omp_dsp_heap_idx idx, int num, int size);
void *__ti_omp_dsp_memalign (ti_omp_dsp_heap_idx idx, int alignment,
                                     int size);

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* OMP_H_ */
