/******************************************************************************
 * Copyright (c) 2013-2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#ifndef TI_OMP_DEVICE_H_
#define TI_OMP_DEVICE_H_

/**
 * \enum __TI_omp_mem_region
 * \brief Contiguous memory (CMEM) regions to create data buffers in
 *
 * TI_OMP_MEM_DDR represents contiguous memory on DDR3 RAM and 
 * TI_OMP_MEM_MSMC represents contiguous memory on MSMC SRAM
 */
typedef enum 
{
    TI_OMP_MEM_DDR  = 0x1,
    TI_OMP_MEM_MSMC = 0x2
}__TI_omp_mem_region;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief Allocate a buffer in the given contiguous memory segment 
 *        with given size and return a host pointer to it  
 * \param device_id Numeric id of the accelerator device 
 * \param mregion   Region of contiguous target memory on which to allocate buffer
 * \param size      Size (in bytes) of buffer to allocate
 * \return Host pointer to allocated buffer on contiguous target memory
 */
void* __TI_omp_device_alloc(int device_id, 
                            __TI_omp_mem_region mregion, 
                            int size
                            );

/**
 * \brief Free a contiguous target memory buffer with the given host pointer
 * \param device_id Numeric id of the accelerator device 
 * \param buffer    Host pointer to target buffer 
 */
void __TI_omp_device_free(int device_id, 
                          void* buffer
                          );

#ifdef __cplusplus
}
#endif

#endif /* TI_TARGET_MALLOC_H_ */
