/******************************************************************************
 * Copyright (c) 2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

/**
 * \file types.h
 * \brief Collection of different types and enums used in the OpenMPAcc Package 
 */

#ifndef TYPES_H_
#define TYPES_H_

#include <iostream>
#include "ti_omp_device.h"

/**
 * \enum MapKind
 * \brief Kinds of OpenMP accelerator model target region maps 
 */
enum MapKind
{
    MAP_ALLOC       = 0x0,
    MAP_TO          = 0x1,
    MAP_FROM        = 0x2,
    MAP_TO_FROM     = 0x3,
    MAP_TO_L2       = 0x4,
    MAP_TO_MSMC     = 0x5
};

/**
 * \enum TargetRegionKind
 * \brief Kinds of target regions
 */
enum TargetRegionKind
{
    TARGET                  = 0x1,
    TARGET_DATA             = 0x2,
    TI_OMP_DEVICE_ALLOC     = 0x3,
    TARGET_UPDATE           = 0x4
};

/**
 * \brief Pretty print MapKind
 * \param os    Output stream reference 
 * \param kind  MapKind value
 * \return Output stream reference
 */
inline std::ostream& operator<<(std::ostream& os, 
                                const MapKind kind
                                )
{
    switch (kind)
    {
        case MAP_TO:
            os << "MAP_TO_DDR";
            break;
        case MAP_FROM:
            os << "MAP_FROM_DDR";
            break;
        case MAP_TO_FROM:
            os << "MAP_TO_FROM_DDR";
            break;
        case MAP_ALLOC:
            os << "MAP_ALLOC";
            break;
        case MAP_TO_MSMC:
            os << "MAP_TO_FROM_MSMC";
            break;
        case MAP_TO_L2:
            os << "MAP_LOCAL";
            break;
        default:
            os << "MAP_KIND_NOT_RECOGNIZED";
            break;
    }
    return os;
}

/**
 * \brief Pretty print TargetRegionKind
 * \param os    Output stream reference 
 * \param kind  TargetRegionKind value
 * \return Output stream reference
 */
inline std::ostream& operator<<(std::ostream& os, 
                                const TargetRegionKind kind
                                )
{
    switch (kind)
    {
        case TARGET:
            os << "TARGET";
            break;
        case TARGET_DATA:
            os << "TARGET_DATA";
            break;
        case TI_OMP_DEVICE_ALLOC:
            os << "TI_OMP_DEVICE_ALLOC";
            break;
        case TARGET_UPDATE:
            os << "TARGET_UPDATE";
            break;
        default:
            os << "TARGET_REGION_KIND_NOT_RECOGNIZED";
            break;
    }
    return os;
}

#endif /* TYPES_H_ */
